@@ -1,10 +1,4 @@
-"""vcsfile cubicweb package : interface local svn repository content as cubicweb
-entities
-
-:organization: Logilab
-:copyright: 2007-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
-"""
+"""cubicweb-vcsfile"""
 
 try:
     from cubicweb import server
