@@ -108,20 +108,45 @@ def set_at_revision(session, reid, safetybelt=False):
     """
     rqlexec = session.unsafe_execute
     # link to version content generated by this revision
-    rql = 'SET VC at_revision REV WHERE REV eid %(rev)s, VC from_revision REV'
+    #rql = 'SET VC at_revision REV WHERE REV eid %(rev)s, VC from_revision REV'
+    #if safetybelt:
+    #    rql += ', NOT VC at_revision REV'
+    #rqlexec(rql, {'rev': reid}, 'rev')
+    sqlrestr = ('SELECT VC.cw_eid, %s FROM cw_%%s AS VC '
+                'WHERE VC.cw_from_revision=%s' % (reid, reid))
     if safetybelt:
-        rql += ', NOT VC at_revision REV'
-    rqlexec(rql, {'rev': reid}, 'rev')
+        sqlrestr += ('AND NOT EXISTS(SELECT 1 FROM at_revision_relation AS atr '
+                     'WHERE atr.eid_to=%s AND atr.eid_from=VC.cw_eid)') % reid
+
+    sql = ('INSERT INTO at_revision_relation(eid_from,eid_to) '
+           '%s UNION %s') % (sqlrestr % 'VersionContent',
+                             sqlrestr % 'DeletedVersionContent')
+    session.system_sql(sql)
     # link to version content of files in parent revisions but not modified by
     # this revision
-    rql = ('SET VC at_revision REV WHERE VC is VersionContent, '
-           'REV eid %(rev)s, VC at_revision PREV, '
-           'REV parent_revision PREV, '
-           'NOT EXISTS(VC2 from_revision REV, VC content_for VF, '
-           'VC2 content_for VF)')
+    # rql = ('SET VC at_revision REV WHERE VC is VersionContent, '
+    #        'REV eid %(rev)s, VC at_revision PREV, '
+    #        'REV parent_revision PREV, '
+    #        'NOT EXISTS(VC2 from_revision REV, VC content_for VF, '
+    #        'VC2 content_for VF)')
+    # if safetybelt:
+    #     rql += ', NOT VC at_revision REV'
+    # rqlexec(rql, {'rev': reid}, 'rev')
+    sql = ('INSERT INTO at_revision_relation(eid_from,eid_to) '
+           'SELECT DISTINCT VC.cw_eid, %s '
+           'FROM at_revision_relation AS ar, parent_revision_relation AS pr, cw_VersionContent AS VC '
+           'WHERE VC.cw_eid=ar.eid_from AND pr.eid_from=%s AND pr.eid_to=ar.eid_to '
+           'AND NOT EXISTS('
+           '  SELECT 1 FROM cw_VersionContent AS VC2 '
+           '  WHERE VC2.cw_from_revision=%s AND VC2.cw_content_for=VC.cw_content_for '
+           '  UNION '
+           '  SELECT 1 FROM cw_DeletedVersionContent AS VC2 '
+           '  WHERE VC2.cw_from_revision=%s AND VC2.cw_content_for=VC.cw_content_for)') % (
+        reid, reid, reid, reid)
     if safetybelt:
-        rql += ', NOT VC at_revision REV'
-    rqlexec(rql, {'rev': reid}, 'rev')
+        sql += (' AND NOT EXISTS(SELECT 1 FROM at_revision_relation AS atr '
+                'WHERE atr.eid_from=%s AND atr.eid_to=VC.cw.cw_eid)') % reid
+    session.system_sql(sql)
 
 
 def import_revision(session, repoeid, date, **kwargs):
@@ -142,14 +167,9 @@ def import_revision(session, repoeid, date, **kwargs):
 
 def import_versioned_file(session, repoeid, date, directory, name):
     """import a version controlled file from a repository"""
-    repohdlr = repository_handler(repoeid)
-    args = {'date': date}
-    args['directory'] = directory
-    args['name'] = name
-    args['repoeid'] = repoeid
+    args = {'directory': directory, 'name': name, 'repoeid': repoeid}
     return session.execute(
         'INSERT VersionedFile X: X directory %(directory)s, '
-        'X creation_date %(date)s, X modification_date %(date)s, '
         'X name %(name)s, X from_repository R '
         'WHERE R eid %(repoeid)s', args)[0][0]
 
@@ -158,18 +178,15 @@ def import_version_content(session, repoeid, reveid, upath, date, **kwargs):
     """import a new file revision from a repository"""
     repohdlr = repository_handler(repoeid)
     vfeid = repohdlr.vf_eid(session, upath, date)
-    args = {'date': date}
-    args['vf'] = vfeid
-    args['rev'] = reveid
     # done in a hook but doing it here as well to avoid an additional query to
     # lookup filename
     mt, enc = guess_mimetype_and_encoding(
         fallbackencoding=repohdlr.encoding, fallbackmimetype=None,
         filename=upath)
-    args['df'] = mt and unicode(mt) or None
-    args['de'] = enc and unicode(enc) or None
+    args = {'vf': vfeid, 'rev': reveid,
+            'df': mt and unicode(mt) or None,
+            'de': enc and unicode(enc) or None}
     rql = ('INSERT VersionContent X: X from_revision REV, X content_for VF, '
-           'X creation_date %(date)s, X modification_date %(date)s, '
            'X data_format %(df)s, X data_encoding %(de)s')
     for inlinerel in ('vc_copy', 'vc_rename'):
         if inlinerel in kwargs:
@@ -183,10 +200,7 @@ def import_deleted_version_content(session, repoeid, reveid, upath, date):
     """import a new deleted file revision from a repository"""
     repohdlr = repository_handler(repoeid)
     vfeid = repohdlr.vf_eid(session, upath, date)
-    args = {'date': date}
-    args['vf'] = vfeid
-    args['rev'] = reveid
-    rql = ('INSERT DeletedVersionContent X: X from_revision REV, X content_for VF, '
-           'X creation_date %(date)s, X modification_date %(date)s '
+    args = {'vf': vfeid, 'rev': reveid}
+    rql = ('INSERT DeletedVersionContent X: X from_revision REV, X content_for VF '
            ' WHERE REV eid %(rev)s, VF eid %(vf)s')
     return session.execute(rql, args)[0][0]
