@@ -7,10 +7,10 @@
 """
 __docformat__ = "restructuredtext en"
 
-from logilab.common.decorators import cached
+from logilab.common.decorators import cached, clear_cache
 from logilab.common.compat import any
 
-from cubicweb import Binary
+from cubicweb import QueryError, Binary
 from cubicweb.interfaces import IPrevNext, IDownloadable
 from cubicweb.entities import AnyEntity, fetch_config
 
@@ -47,6 +47,10 @@ class Repository(AnyEntity):
             title += ' (%s)' % self.subpath
         return title
 
+    def clear_all_caches(self):
+        super(VersionedFile, self).clear_all_caches()
+        clear_cache(self, 'branch_head')
+
     # navigation in versioned content #########################################
 
     def branches(self):
@@ -103,20 +107,18 @@ class Repository(AnyEntity):
         if author is _MARKER:
             author = self.req.user.login
         parent = self.branch_head(branch)
-        if parent is None:
+        if parent is None and branch != parentbranch:
             parent = self.branch_head(parentbranch)
-        assert parent
+        #assert parent
         args = {'description' : msg, 'author': author, 'branch': branch,
-                'repoeid' : self.eid, 'parent': parent.eid}
-        rset = self.req.execute(queries.new_revision_rql(True), args)
+                'repoeid' : self.eid, 'parent': parent and parent.eid}
+        rset = self.req.execute(queries.new_revision_rql(parent), args)
         return rset.get_entity(0, 0)
 
     def vcs_add(self, filedir, filename, stream, **kwargs):
-        vf = self.req.execute(
-            'INSERT VersionedFile X: X from_repository R, X directory %(dir)s, '
-            'X name %(name)s WHERE R eid %(repo)s ',
-            {'repo': self.eid, 'dir': filedir,
-             'name': filename}).get_entity(0, 0)
+        vf = self.req.execute(queries.new_file_rql(),
+                              {'repoeid': self.eid, 'directory': filedir,
+                               'name': filename}).get_entity(0, 0)
         vf.vcs_upload_revision(stream, **kwargs)
         return vf.eid
 
@@ -139,6 +141,11 @@ class Revision(AnyEntity):
         return self.req._('%(rev)s of repository %(repo)s') % {
             'rev': self.dc_title(), 'repo': self.repository.dc_title()}
 
+    def sortvalue(self, rtype=None):
+        if rtype is None:
+            return self.revision
+        return super(Revision, self).sortvalue(rtype)
+
     def previous_entity(self): # IPrevNext
         # may have multiple parents, take the first one in the same branch...
         rset = self.req.execute('Any X LIMIT 1 WHERE R parent_revision X , '
@@ -158,9 +165,6 @@ class Revision(AnyEntity):
     def parent(self): # for breadcrumbs
         return self.repository
 
-    def add_related_schemas(self): # disable automatic add related actions
-        return []
-
     def pre_web_edit(self):
         """callback called by the web editcontroller when an entity will be
         created/modified, to let a chance to do some entity specific stuff.
@@ -198,8 +202,10 @@ class VersionedFile(AnyEntity):
     def parent(self): # for breadcrumbs
         return self.repository
 
-    def add_related_schemas(self): # disable automatic add related actions
-        return []
+    def clear_all_caches(self):
+        super(VersionedFile, self).clear_all_caches()
+        clear_cache(self, 'version_content')
+        clear_cache(self, 'branch_head')
 
     # navigation in versioned content #########################################
 
@@ -275,33 +281,34 @@ class VersionedFile(AnyEntity):
     # vcs write support #######################################################
 
     def vcs_rm(self, rev=None, **kwargs):
-        """either takes a rev or a kwargs having keys :
-        branch, msg, author
+        """remove this file from the vcs
+
+        takes a rev or a kwargs having keys: branch, msg, author
         """
+        if not isinstance(self.branch_head(kwargs.get('branch', _MARKER)),
+                          VersionContent):
+            # already deleted
+            raise QueryError(self.req._('%s is already deleted from the vcs')
+                             % self.path)
         if rev is None:
             if not 'msg' in kwargs:
                 kwargs['msg'] = self.req._('deleted %s') % self.dc_title()
             rev = self.repository.make_revision(**kwargs)
-        yet = self.req.execute('DeletedVersionContent D WHERE D content_for X, '
-                               'X eid %(x)s', {'x' : self.eid}, 'x')
-        if not len(yet):
-            self.req.execute(
-                'INSERT DeletedVersionContent D: D content_for X, '
-                'D from_revision R WHERE X eid %(x)s, R eid %(r)s',
-                {'x' : self.eid, 'r' : rev.eid}, ('x', 'r'))
+        self.req.execute(
+            'INSERT DeletedVersionContent D: D content_for X, '
+            'D from_revision R WHERE X eid %(x)s, R eid %(r)s',
+            {'x' : self.eid, 'r' : rev.eid}, ('x', 'r'))
 
     def vcs_upload_revision(self, stream, rev=None, **kwargs):
-        """either takes a rev or a kwargs having keys :
-        branch, msg, author
+        """upload a new revision for this file to the vcs
+
+        either takes a rev or a kwargs having keys: branch, msg, author
         """
         if rev is None:
             rev = self.repository.make_revision(**kwargs)
-        return self.req.execute(
-            'INSERT VersionContent X: '
-            'X content_for VF, X from_revision REV, X data %(data)s '
-            'WHERE VF eid %(vfeid)s, REV eid %(rev)s',
-            {'vfeid' : self.eid, 'rev' : rev.eid,
-             'data' : Binary(stream.read())})[0][0]
+        return self.req.execute(queries.new_file_content_rql(),
+                                {'vfeid' : self.eid, 'reveid' : rev.eid,
+                                 'data' : Binary(stream.read())})[0][0]
 
 
 class DeletedVersionContent(AnyEntity):
