@@ -232,15 +232,31 @@ class AddParentRevisionHook(Hook):
     accepts = ('parent_revision',)
 
     def call(self, session, fromeid, rtype, toeid):
-        session.repo.glob_add_relation(session,
-                                       fromeid, 'ancestor_revision', toeid)
+        # ancestor relation may already be set
+        # if session.unsafe_execute('Any X WHERE X eid %(x)s, Y eid %(y)s, '
+        #                           'X ancestor_revision Y',
+        #                           {'x': fromeid, 'y': toeid}):
+        #     return
+        # session.repo.glob_add_relation(session,
+        #                                fromeid, 'ancestor_revision', toeid)
         # need NOT safety belt in case of merge nodes with two parents
-        session.unsafe_execute(
-            'SET R ancestor_revision AR '
-            'WHERE R eid %(rev)s, PR eid %(prev)s, PR ancestor_revision AR, '
-            'NOT R ancestor_revision AR',
-            {'rev': fromeid, 'prev': toeid}, 'rev')
-
+        # session.unsafe_execute(
+        #     'SET R ancestor_revision AR '
+        #     'WHERE R eid %(rev)s, PR eid %(prev)s, PR ancestor_revision AR, '
+        #     'NOT R ancestor_revision AR',
+        #     {'rev': fromeid, 'prev': toeid}, 'rev')
+        session.system_sql('INSERT INTO ancestor_revision_relation(eid_from,eid_to) '
+                           'SELECT %s, %s '
+                           'WHERE NOT EXISTS('
+                           '  SELECT 1 FROM ancestor_revision_relation AS ar '
+                           '  WHERE ar.eid_from=%s AND ar.eid_to=%s'
+                           ')' % (fromeid, toeid, fromeid, toeid))
+        session.system_sql('INSERT INTO ancestor_revision_relation(eid_from,eid_to) '
+                           'SELECT %s, ar.eid_to FROM ancestor_revision_relation AS ar '
+                           'WHERE ar.eid_from=%s AND NOT EXISTS('
+                           '  SELECT 1 FROM ancestor_revision_relation AS ar2 '
+                           '  WHERE ar2.eid_from=%s AND ar2.eid_to=ar.eid_to'
+                           ')' % (fromeid, toeid, fromeid))
 
 # safety belts #################################################################
 
