@@ -32,7 +32,7 @@ class SVNRepository(repo.VCSRepository):
         repos.open(fspath) # throwaway, just check it's a valid svn repository
         return fspath
 
-    def import_content(self, repoentity):
+    def import_content(self, repoentity, commitevery=10):
         """import content from the subversion repository
 
         NOTE: server side only method
@@ -47,19 +47,31 @@ class SVNRepository(repo.VCSRepository):
         #ra_ctx = ra.open2('file://' + self.path, callbacks, None, None)
         latestrev = rev = repoentity.latest_known_revision
         actualrev = fs.youngest_rev(fsrep)
-        execute = repoentity.req.execute
+        session = repoentity.req
         while rev < actualrev:
             base_root = fs.revision_root(fsrep, rev)
             rev += 1
-            if execute('Any X WHERE X from_repository R, R eid %(repo)s, X revision %(rev)s',
-                       {'repo': repoentity.eid, 'rev': rev}):
+            if session.execute('Any X WHERE X from_repository R, R eid %(repo)s,'
+                               'X revision %(rev)s',
+                               {'repo': repoentity.eid, 'rev': rev}):
                 self.warning('skip revision %s, seems already imported', rev)
                 continue
             self.info('importing content of %s for revision %s', self, rev)
             self.imported_revision = rev # for hooks
             # the base of the comparison
             editor = SVNImporter(repoentity, fsrep, base_root, rev)
-            editor.import_revision()
+            try:
+                editor.import_revision()
+            except:
+                self.critical('error while importing revision %s of %s',
+                              rev, repoentity.path)
+                raise
+            if not rev % commitevery:
+                repoentity.set_attributes(latest_known_revision=rev)
+                # give a change to threads waiting for a pool by freeing it then
+                # reacquiring it
+                session.commit()
+                session.set_pool()
         if actualrev > latestrev:
             repoentity.set_attributes(latest_known_revision=actualrev)
 
