@@ -7,14 +7,25 @@
 __docformat__ = "restructuredtext en"
 _ = unicode
 
-from logilab.mtconverter import TransformError, html_escape
+from logilab.mtconverter import TransformError, xml_escape
 
 from cubicweb.selectors import implements
+from cubicweb.view import EntityView
 from cubicweb.common import mttransforms, tags
 from cubicweb.web import uicfg
-from cubicweb.web.views.idownloadable import IDownloadablePrimaryView
-from cubicweb.web.views.baseviews import MetaDataView
-from cubicweb.web.views.primary import PrimaryView
+from cubicweb.web.views import (ibreadcrumbs, idownloadable,
+                                baseviews, primary, tabs)
+
+_pvs = uicfg.primaryview_section
+_pvs.tag_attribute(('VersionedFile', 'name'), 'hidden')
+_pvs.tag_attribute(('VersionedFile', 'directory'), 'hidden')
+_pvs.tag_subject_of(('*', 'ancestor_revision', '*'), 'hidden')
+_pvs.tag_object_of(('*', 'from_repository', '*'), 'hidden')
+for etype in ('DeletedVersionContent', 'VersionContent'):
+    _pvs.tag_attribute((etype, 'revision'), 'hidden')
+    _pvs.tag_attribute((etype, 'author'), 'hidden')
+    _pvs.tag_subject_of((etype, 'content_for', '*'), 'hidden')
+
 
 def render_entity_summary(self, entity):
     if entity.description:
@@ -23,38 +34,80 @@ def render_entity_summary(self, entity):
         msg = self.req._('this file has newer revisions')
         self.w(tags.div(msg, klass='warning'))
 
-for etype in ('DeletedVersionContent', 'VersionContent'):
-    uicfg.primaryview_section.tag_attribute((etype, 'revision'), 'hidden')
-    uicfg.primaryview_section.tag_attribute((etype, 'author'), 'hidden')
-    uicfg.primaryview_section.tag_subject_of((etype, 'content_for', '*'), 'hidden')
 
-uicfg.primaryview_section.tag_attribute(('VersionedFile', 'name'), 'hidden')
-uicfg.primaryview_section.tag_attribute(('VersionedFile', 'directory'), 'hidden')
+class RepositoryPrimaryView(tabs.TabsMixin, primary.PrimaryView):
+    __select__ = implements('Repository')
+
+    tabs = [_('repositoryinfo_tab'), _('repositoryfiles_tab'), _('repositoryhistory_tab')]
+    default_tab = 'repositoryinfo_tab'
+
+    def cell_call(self, row, col):
+        entity = self.complete_entity(row, col)
+        self.render_entity_title(entity)
+        self.render_tabs(self.tabs, self.default_tab, entity)
 
 
-class DVCPrimaryView(PrimaryView):
+class RepositoryInfoTab(primary.PrimaryView):
+    id = 'repositoryinfo_tab'
+    __select__ = implements('Repository')
+
+    def render_entity_title(self, entity):
+        pass
+
+    def render_entity_attributes(self, entity):
+        super(RepositoryInfoTab, self).render_entity_attributes(entity)
+        rset = self.req.execute('Any REV, REVB, REVN ORDERBY REVN DESC '
+                                'WHERE REV branch REVB, REV revision REVN, REV from_repository R, '
+                                'R eid %(r)s, NOT X parent_revision REV',
+                                {'r': entity.eid}, 'r')
+        if rset:
+            self.w('<h3>%s</h3>' % self.req._('heads'))
+            self.wview('table', rset)
+
+
+class RepositoryFilesTab(EntityView):
+    id = 'repositoryfiles_tab'
+    __select__ = implements('Repository')
+
+    def cell_call(self, row, col):
+        entity = self.rset.get_entity(row, col)
+        filesrql = entity.related_rql('from_repository', 'object', ('VersionedFile',))
+        self.wview('list', self.req.execute(filesrql, {'x': entity.eid}, 'x'), 'noresult')
+
+
+class RepositoryHistoryTab(EntityView):
+    id = 'repositoryhistory_tab'
+    __select__ = implements('Repository')
+
+    def cell_call(self, row, col):
+        entity = self.rset.get_entity(row, col)
+        revisionsrql = entity.related_rql('from_repository', 'object', ('Revision',))
+        self.wview('table', self.req.execute(revisionsrql, {'x': entity.eid}, 'x'), 'noresult')
+
+
+class DVCPrimaryView(primary.PrimaryView):
     __select__ = implements('DeletedVersionContent')
 
     def render_entity_title(self, entity):
-        self.w(self.req._('Revision %(revision)s of %(file)s: DELETED')
-               % {'revision': entity.revision,
-                  'file': entity.file.view('oneline')})
+        title = self.req._('Revision %(revision)s of %(file)s: DELETED') % {
+            'revision': entity.revision, 'file': entity.file.view('oneline')}
+        self.w('<h1>%s</h1>' % title)
 
     render_entity_summary = render_entity_summary
 
 
-class VCPrimaryView(IDownloadablePrimaryView):
+class VCPrimaryView(idownloadable.IDownloadablePrimaryView):
     __select__ = implements('VersionContent')
 
     def render_entity_title(self, entity):
-        self.w(self.req._('Revision %(revision)s of %(file)s')
-               % {'revision': entity.revision,
-                  'file': entity.file.view('oneline')})
+        title = self.req._('Revision %(revision)s of %(file)s') % {
+            'revision': entity.revision, 'file': entity.file.view('oneline')}
+        self.w('<h1>%s</h1>' % title)
 
     render_entity_summary = render_entity_summary
 
 
-class VCMetaDataView(MetaDataView):
+class VCMetaDataView(baseviews.MetaDataView):
     """paragraph view of some metadata"""
     __select__ = implements('VersionContent', 'DeletedVersionContent')
 
@@ -65,7 +118,7 @@ class VCMetaDataView(MetaDataView):
         self.w(u'#%s - ' % entity.eid)
         self.w(u'<span>%s</span> ' % _('revision %s of') % entity.revision)
         self.w(u'<span class="value">%s</span>,&nbsp;'
-               % entity.file.path)
+               % xml_escape(entity.file.path))
         self.w(u'<span>%s</span> ' % _('created on'))
         self.w(u'<span class="value">%s</span>'
                % self.format_date(entity.creation_date))
@@ -75,7 +128,7 @@ class VCMetaDataView(MetaDataView):
         self.w(u'</div>')
 
 
-class VFPrimaryView(PrimaryView):
+class VFPrimaryView(primary.PrimaryView):
     __select__ = implements('VersionedFile')
 
     def render_entity_attributes(self, entity):
@@ -96,7 +149,7 @@ class VFPrimaryView(PrimaryView):
                     pass
                 except Exception, ex:
                     msg = self.req._("can't display data, unexpected error: %s") % ex
-                    self.w('<div class="error">%s</div>' % msg)
+                    self.w('<div class="error">%s</div>' % xml_escape(msg))
         self.w(u'</div>')
 
     def render_entity_relations(self, entity):
@@ -106,20 +159,20 @@ class VFPrimaryView(PrimaryView):
                (_('revision'), _('author'), _('comment')))
         for vc in reversed(revisions):
             self.w(u'<tr><td><a href="%s">%s</a></td><td>%s</td><td>%s</td></tr>\n'
-                   % (html_escape(vc.absolute_url()), vc.revision,
-                      html_escape(vc.author or u''),
-                      html_escape(vc.description or u'')))
+                   % (xml_escape(vc.absolute_url()), vc.revision,
+                      xml_escape(vc.author or u''),
+                      xml_escape(vc.description or u'')))
         self.w(u'</table>\n')
 
 
-class RevisionPrimaryView(PrimaryView):
+class RevisionPrimaryView(primary.PrimaryView):
     __select__ = implements('Revision')
 
     def render_entity_attributes(self, rev):
-        self.field('description', html_escape(rev.description))
-        self.field('author', html_escape(rev.author))
+        self.field('description', xml_escape(rev.description))
+        self.field('author', xml_escape(rev.author))
         if rev.branch:
-            self.field('branch', html_escape(rev.branch))
+            self.field('branch', xml_escape(rev.branch))
         self.field('from_repository', rev.repository.dc_title())
         parents = rev.related('parent_revision')
         if parents:
@@ -136,7 +189,17 @@ class RevisionPrimaryView(PrimaryView):
                    self.req._('objects concerned by this revision'))
             for obj in versioned:
                 self.w(u'<tr><td><a href="%s">%s</a></td></tr>' % (
-                    obj.absolute_url(), html_escape(obj.dc_title())))
+                    obj.absolute_url(), xml_escape(obj.dc_title())))
             self.w(u'</table>')
         else:
             self.w(u'<p>%s</p>' % self.req._('this revision hasn\'t modified any files'))
+
+
+
+class DVCBreadCrumbView(ibreadcrumbs.BreadCrumbView):
+    id = 'breadcrumbtext'
+    __select__ = implements('VersionContent', 'DeletedVersionContent')
+
+    def cell_call(self, row, col):
+        entity = self.rset.get_entity(row, col)
+        self.w(entity.rev.dc_title())
